/*****************************************************************************
* | File      	:	OLED_Driver.c
* | Author      :   Waveshare team
* | Function    :	2.23inch OLED using SSD1306 control
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2019-07-31
* | Info        :   Basic version
******************************************************************************/
#include "OLED_Driver.h"
#include <stdio.h>
#include "Show_Lib.h"
#include "string.h"

/*******************************************************************************
function:
		Common register initialization
*******************************************************************************/
static void OLED_InitReg()
{
	OLED_WriteReg(0xAE);//--turn off oled panel
    OLED_WriteReg(0x04);//--Set Lower Column Start Address for Page Addressing Mode	
    OLED_WriteReg(0x10);//--Set Higher Column Start Address for Page Addressing Mode
    OLED_WriteReg(0x40);//---Set Display Start Line
    OLED_WriteReg(0x81);//---Set Contrast Control for BANK0
    OLED_WriteReg(0x80);//--Contrast control register is set
    OLED_WriteReg(0xA1);//--Set Segment Re-map
    OLED_WriteReg(0xA6);//--Set Normal/Inverse Display
    OLED_WriteReg(0xA8);//--Set Multiplex Ratio
    OLED_WriteReg(0x1F);
    OLED_WriteReg(0xC8);//--Set COM Output Scan Direction
    OLED_WriteReg(0xD3);//--Set Display Offset
    OLED_WriteReg(0x00);
    OLED_WriteReg(0xD5);//--Set Display Clock Divide Ratio/ Oscillator Frequency
    OLED_WriteReg(0xF0);
    OLED_WriteReg(0xD8);//--Set Area Color Mode ON/OFF & Low Power Display Mode
    OLED_WriteReg(0x05);
    OLED_WriteReg(0xD9);//--Set pre-charge period
    OLED_WriteReg(0xC2);
    OLED_WriteReg(0xDA);//--Set COM Pins Hardware Configuration
    OLED_WriteReg(0x12);
    OLED_WriteReg(0xDB);//--Set VCOMH Deselect Level
    OLED_WriteReg(0x08);//--Set VCOM Deselect Level
    OLED_WriteReg(0xAF);//--Normal Brightness Display ON
}

void OLED_Clear(void)
{
    char Column,Page;
    for(Page = 0; Page < OLED_Page; Page++) {
        OLED_WriteReg(0xb0 + Page);    //Set page address
        OLED_WriteReg(0x04);           //Set display position - column low address
        OLED_WriteReg(0x10);           //Set display position - column high address
        for(Column = 0; Column < OLED_Column; Column++)
            OLED_WriteData(0xff);
    }
}

void OLED_Init(void)
{
    OLED_InitReg();
	
	OLED_Clear();
}

void OLED_SendBuf(UBYTE *Buf)
{
    char Column,Page;
    UBYTE *ptr = Buf;
    for(Page = 0; Page < OLED_Page; Page++) {
        OLED_WriteReg(0xb0 + Page);
        OLED_WriteReg(0x04);
        OLED_WriteReg(0x10);
        for(Column = 0; Column < OLED_Column; Column++) {
            OLED_WriteData(*ptr);
            ptr++;
        }
    } 
}
