#include <bcm2835.h>
#include <stdio.h>
#include "ssd1305.h"
#define VCCSTATE SSD1305_SWITCHCAPVCC
#define WIDTH 128
#define HEIGHT 32
#define PAGES 8

#define RST 25 
#define DC  24
char buffer[512];

void command(char cmd) {
    bcm2835_gpio_write(DC, LOW);
    bcm2835_spi_transfer(cmd);
}

void SSD1305_begin()
{
    bcm2835_gpio_fsel(RST, BCM2835_GPIO_FSEL_OUTP);
    bcm2835_gpio_fsel(DC, BCM2835_GPIO_FSEL_OUTP);

    bcm2835_spi_begin();
    bcm2835_spi_setBitOrder(BCM2835_SPI_BIT_ORDER_MSBFIRST);     //The default
    bcm2835_spi_setDataMode(BCM2835_SPI_MODE0);                  //The default
    bcm2835_spi_setClockDivider(BCM2835_SPI_CLOCK_DIVIDER_2048);  //The default
    bcm2835_spi_chipSelect(BCM2835_SPI_CS0);                     //The default
    bcm2835_spi_setChipSelectPolarity(BCM2835_SPI_CS0, LOW);     //the default

    bcm2835_gpio_write(RST, HIGH);
    bcm2835_delay(10);
    bcm2835_gpio_write(RST, LOW);
    bcm2835_delay(10);
    bcm2835_gpio_write(RST, HIGH);
  
    command(0xAE);//--turn off oled panel
    command(0x04);//--Set Lower Column Start Address for Page Addressing Mode	
    command(0x10);//--Set Higher Column Start Address for Page Addressing Mode
    command(0x40);//---Set Display Start Line
    command(0x81);//---Set Contrast Control for BANK0
    command(0x80);//--Contrast control register is set
    command(0xA1);//--Set Segment Re-map
    command(0xA6);//--Set Normal/Inverse Display
    command(0xA8);//--Set Multiplex Ratio
    command(0x1F);
    command(0xC8);//--Set COM Output Scan Direction
    command(0xD3);//--Set Display Offset
    command(0x00);
    command(0xD5);//--Set Display Clock Divide Ratio/ Oscillator Frequency
    command(0xF0);
    command(0xD8);//--Set Area Color Mode ON/OFF & Low Power Display Mode
    command(0x05);
    command(0xD9);//--Set pre-charge period
    command(0xC2);
    command(0xDA);//--Set COM Pins Hardware Configuration
    command(0x12);
    command(0xDB);//--Set VCOMH Deselect Level
    command(0x08);//--Set VCOM Deselect Level
    command(0xAF);//--Normal Brightness Display ON
}
void SSD1305_clear()
{
    int i;
    for(i = 0; i < sizeof(buffer); i++)
    {
        buffer[i] = 0;
    }
}
void SSD1305_pixel(int x, int y, char color)
{
    if(x > WIDTH || y > HEIGHT)return ;
    if(color)
        buffer[x+(y/8)*WIDTH] |= 1<<(y%8);
    else
        buffer[x+(y/8)*WIDTH] &= ~(1<<(y%8));
}
void SSD1305_char1616(uint8_t x, uint8_t y, uint8_t chChar)
{
    uint8_t i, j;
    uint8_t chTemp = 0, y0 = y, chMode = 0;

    for (i = 0; i < 32; i ++) {
        chTemp = Font1612[chChar - 0x30][i];
        for (j = 0; j < 8; j ++) {
            chMode = chTemp & 0x80? 1 : 0; 
            SSD1305_pixel(x, y, chMode);
            chTemp <<= 1;
            y ++;
            if ((y - y0) == 16) {
                y = y0;
                x ++;
                break;
            }
        }
    }
}

void SSD1305_char3216(uint8_t x, uint8_t y, uint8_t chChar)
{
    uint8_t i, j;
    uint8_t chTemp = 0, y0 = y, chMode = 0;

    for (i = 0; i < 64; i ++) {
        chTemp = Font3216[chChar - 0x30][i];
        for (j = 0; j < 8; j ++) {
            chMode = chTemp & 0x80? 1 : 0; 
            SSD1305_pixel(x, y, chMode);
            chTemp <<= 1;
            y ++;
            if ((y - y0) == 32) {
                y = y0;
                x ++;
                break;
            }
        }
    }
}
void SSD1305_char(unsigned char x, unsigned char y, char acsii, char size, char mode)
{
    unsigned char i, j, y0=y;
    char temp;
    unsigned char ch = acsii - ' ';
    for(i = 0;i<size;i++) {
        if(size == 12)
        {
            if(mode)temp=Font1206[ch][i];
            else temp = ~Font1206[ch][i];
        }
        else 
        {            
            if(mode)temp=Font1608[ch][i];
            else temp = ~Font1608[ch][i];
        }
        for(j =0;j<8;j++)
        {
            if(temp & 0x80) SSD1305_pixel(x, y, 1);
            else SSD1305_pixel(x, y, 0);
            temp <<=1;
            y++;
            if((y-y0)==size)
            {
                y = y0;
                x ++;
                break;
            }
        }
    }
}
void SSD1305_string(uint8_t x, uint8_t y, const char *pString, uint8_t Size, uint8_t Mode)
{
    while (*pString != '\0') {       
        if (x > (WIDTH - Size / 2)) {
            x = 0;
            y += Size;
            if (y > (HEIGHT - Size)) {
                y = x = 0;
            }
        }
        
        SSD1305_char(x, y, *pString, Size, Mode);
        x += Size / 2;
        pString ++;
    }
}

void SSD1305_bitmap(unsigned char x, unsigned char y, const unsigned char *pBmp, 
                        unsigned char chWidth, unsigned char chHeight)
{
    unsigned char i, j, byteWidth = (chWidth + 7)/8;
    for(j = 0; j < chHeight; j++) {
        for(i = 0;i <chWidth;i ++) {
            if(*(pBmp + j*byteWidth + i/8) & (128 >> (i & 7))) {
                SSD1305_pixel(x+i, y+j, 1);
            }
        }
    }        
}

void SSD1305_display()
{
    uint8_t page;
    char *pBuf = (char *)buffer;
    
    for (page = 0; page < 8; page++) {  
        /* set page address */
        command(0xB0 + page);
        /* set low column address */
        command(0x04); 
        /* set high column address */
        command(0x10); 
        /* write data */
        bcm2835_gpio_write(DC, HIGH);
        bcm2835_spi_transfern(pBuf, WIDTH); 
        pBuf += WIDTH;
    }
}
